@@ -135,7 +135,7 @@ class CommonTestMixin_v4(CommonTestMixin):
 
     def test_bad_packed_length(self):
         def assertBadLength(length):
-            addr = b'0' * length
+            addr = b'\0' * length
             msg = "%r (len %d != 4) is not permitted as an IPv4 address"
             with self.assertAddressError(re.escape(msg % (addr, length))):
                 self.factory(addr)
@@ -155,11 +155,11 @@ class CommonTestMixin_v6(CommonTestMixin):
         self.assertInstancesEqual(3232235521, "::c0a8:1")
 
     def test_packed(self):
-        addr = (b'\0' * 12) + _compat_bytes_fromhex("00000000")
+        addr = b'\0'*12 + _compat_bytes_fromhex("00000000")
         self.assertInstancesEqual(addr, "::")
-        addr = (b'\0' * 12) + _compat_bytes_fromhex("c0a80001")
+        addr = b'\0'*12 + _compat_bytes_fromhex("c0a80001")
         self.assertInstancesEqual(addr, "::c0a8:1")
-        addr = _compat_bytes_fromhex("c0a80001") + (b'\0' * 12)
+        addr = _compat_bytes_fromhex("c0a80001") + b'\0'*12
         self.assertInstancesEqual(addr, "c0a8:1::")
 
     def test_negative_ints_rejected(self):
@@ -174,7 +174,7 @@ class CommonTestMixin_v6(CommonTestMixin):
 
     def test_bad_packed_length(self):
         def assertBadLength(length):
-            addr = b'0' * length
+            addr = b'\0' * length
             msg = "%r (len %d != 16) is not permitted as an IPv6 address"
             with self.assertAddressError(re.escape(msg % (addr, length))):
                 self.factory(addr)
@@ -523,7 +523,7 @@ class NetworkTestCase_v4(BaseTestCase, NetmaskTestMixin_v4):
                 self.factory('10.0.1.0/24')))
         # containee larger than container
         self.assertFalse(
-            self.factory('10.0.0.0/24').subnet_of(
+            self.factory('10.0.1.0/24').subnet_of(
                 self.factory('10.0.0.0/30')))
 
     def test_supernet_of(self):
@@ -544,6 +544,28 @@ class NetworkTestCase_v4(BaseTestCase, NetmaskTestMixin_v4):
             self.factory('10.0.0.0/24').supernet_of(
                 self.factory('10.0.0.0/30')))
 
+    def test_subnet_of_mixed_types(self):
+        self.assertRaises(
+            TypeError,
+            ipaddress.IPv4Network('10.0.0.0/30').supernet_of,
+            ipaddress.IPv6Network('::1/128'),
+        )
+        self.assertRaises(
+            TypeError,
+            ipaddress.IPv6Network('::1/128').subnet_of,
+            ipaddress.IPv4Network('10.0.0.0/30'),
+        )
+        self.assertRaises(
+            TypeError,
+            ipaddress.IPv4Network('10.0.0.0/30').subnet_of,
+            ipaddress.IPv6Network('::1/128'),
+        )
+        self.assertRaises(
+            TypeError,
+            ipaddress.IPv6Network('::1/128').subnet_of,
+            ipaddress.IPv4Network('10.0.0.0/30'),
+        )
+
 
 class NetmaskTestMixin_v6(CommonTestMixin_v6):
     """Input validation on interfaces and networks is very similar"""
@@ -725,8 +747,8 @@ class ComparisonTests(unittest.TestCase):
     def test_same_type_equality(self):
         for obj in self.objects:
             self.assertEqual(obj, obj)
-            self.assertLessEqual(obj, obj)
-            self.assertGreaterEqual(obj, obj)
+            self.assertTrue(obj <= obj)
+            self.assertTrue(obj >= obj)
 
     def test_same_type_ordering(self):
         for lhs, rhs in (
@@ -738,10 +760,10 @@ class ComparisonTests(unittest.TestCase):
             (self.v6intf, self.v6intf2),
         ):
             self.assertNotEqual(lhs, rhs)
-            self.assertLess(lhs, rhs)
-            self.assertLessEqual(lhs, rhs)
-            self.assertGreater(rhs, lhs)
-            self.assertGreaterEqual(rhs, lhs)
+            self.assertTrue(lhs < rhs)
+            self.assertTrue(lhs <= rhs)
+            self.assertTrue(rhs > lhs)
+            self.assertTrue(rhs >= lhs)
             self.assertFalse(lhs > rhs)
             self.assertFalse(rhs < lhs)
             self.assertFalse(lhs >= rhs)
@@ -840,9 +862,9 @@ class IpaddrUnitTest(unittest.TestCase):
         self.ipv6_network = ipaddress.IPv6Network('2001:658:22a:cafe::/64')
 
     def testRepr(self):
-        self.assertTrue(re.match("IPv4Interface\(u?'1.2.3.4/32'\)",
+        self.assertTrue(re.match(r"IPv4Interface\(u?'1.2.3.4/32'\)",
                         repr(ipaddress.IPv4Interface('1.2.3.4'))))
-        self.assertTrue(re.match("IPv6Interface\(u?'::1/128'\)",
+        self.assertTrue(re.match(r"IPv6Interface\(u?'::1/128'\)",
                         repr(ipaddress.IPv6Interface('::1'))))
 
     # issue #16531: constructing IPv4Network from an (address, mask) tuple
@@ -992,7 +1014,8 @@ class IpaddrUnitTest(unittest.TestCase):
                          [(ip1, ip3)])
         self.assertEqual(128, ipaddress._count_righthand_zero_bits(0, 128))
         self.assertTrue(
-            re.match("IPv4Network\(u?'1.2.3.0/24'\)", repr(self.ipv4_network)))
+            re.match(r"IPv4Network\(u?'1.2.3.0/24'\)",
+                     repr(self.ipv4_network)))
 
     def testMissingAddressVersion(self):
         class Broken(ipaddress._BaseAddress):
@@ -1543,14 +1566,35 @@ class IpaddrUnitTest(unittest.TestCase):
                         ipaddress.ip_address('::2'))
 
     def testInterfaceComparison(self):
-        self.assertTrue(ipaddress.ip_interface('1.1.1.1') <=
-                        ipaddress.ip_interface('1.1.1.1'))
-        self.assertTrue(ipaddress.ip_interface('1.1.1.1') <=
-                        ipaddress.ip_interface('1.1.1.2'))
-        self.assertTrue(ipaddress.ip_interface('::1') <=
-                        ipaddress.ip_interface('::1'))
-        self.assertTrue(ipaddress.ip_interface('::1') <=
-                        ipaddress.ip_interface('::2'))
+        self.assertTrue(ipaddress.ip_interface('1.1.1.1/24') ==
+                        ipaddress.ip_interface('1.1.1.1/24'))
+        self.assertTrue(ipaddress.ip_interface('1.1.1.1/16') <
+                        ipaddress.ip_interface('1.1.1.1/24'))
+        self.assertTrue(ipaddress.ip_interface('1.1.1.1/24') <
+                        ipaddress.ip_interface('1.1.1.2/24'))
+        self.assertTrue(ipaddress.ip_interface('1.1.1.2/16') <
+                        ipaddress.ip_interface('1.1.1.1/24'))
+        self.assertTrue(ipaddress.ip_interface('1.1.1.1/24') >
+                        ipaddress.ip_interface('1.1.1.1/16'))
+        self.assertTrue(ipaddress.ip_interface('1.1.1.2/24') >
+                        ipaddress.ip_interface('1.1.1.1/24'))
+        self.assertTrue(ipaddress.ip_interface('1.1.1.1/24') >
+                        ipaddress.ip_interface('1.1.1.2/16'))
+
+        self.assertTrue(ipaddress.ip_interface('::1/64') ==
+                        ipaddress.ip_interface('::1/64'))
+        self.assertTrue(ipaddress.ip_interface('::1/64') <
+                        ipaddress.ip_interface('::1/80'))
+        self.assertTrue(ipaddress.ip_interface('::1/64') <
+                        ipaddress.ip_interface('::2/64'))
+        self.assertTrue(ipaddress.ip_interface('::2/48') <
+                        ipaddress.ip_interface('::1/64'))
+        self.assertTrue(ipaddress.ip_interface('::1/80') >
+                        ipaddress.ip_interface('::1/64'))
+        self.assertTrue(ipaddress.ip_interface('::2/64') >
+                        ipaddress.ip_interface('::1/64'))
+        self.assertTrue(ipaddress.ip_interface('::1/64') >
+                        ipaddress.ip_interface('::2/48'))
 
     def testNetworkComparison(self):
         # ip1 and ip2 have the same network address
@@ -2087,6 +2131,7 @@ class IpaddrUnitTest(unittest.TestCase):
                          sixtofouraddr.sixtofour)
         self.assertFalse(bad_addr.sixtofour)
 
+
 # Monkey-patch test runner
 if not hasattr(BaseTestCase, 'assertRaisesRegex'):
     class _AssertRaisesRegex(object):
@@ -2117,7 +2162,7 @@ if not hasattr(BaseTestCase, 'assertRaisesRegex'):
 
             expected_regex = self.expected_regex
             if not expected_regex.search(str(exc_value)):
-                raise AssertionError('"{}" does not match "{}"'.format(
+                raise AssertionError('"{0}" does not match "{1}"'.format(
                     expected_regex.pattern, str(exc_value)))
             return True
 
@@ -2168,7 +2213,7 @@ class CompatTest(unittest.TestCase):
         self.assertEqual(ipaddress._compat_bit_length(4), 3)
 
 
-class SingleIssuesTest(unittest.TestCase):
+class SingleIssuesTest(BaseTestCase):
     # https://github.com/phihag/ipaddress/issues/14
     def test_issue_14(self):
         self.assertTrue(ipaddress.ip_address('127.0.0.1').is_private)
@@ -2181,5 +2226,12 @@ class SingleIssuesTest(unittest.TestCase):
         self.assertTrue(net2.subnet_of(net1))
         self.assertFalse(net2.supernet_of(net1))
 
+    def test_issue_48(self):
+        v6net = ipaddress.ip_network('::/0')
+        v4net = ipaddress.ip_network('1.2.3.0/24')
+        with self.assertRaisesRegex(TypeError, r'are not of the same version'):
+            v6net.subnet_of(v4net)
+
+
 if __name__ == '__main__':
     unittest.main()
